#ifndef _COFPCDAT_H_ 
#define _COFPCDAT_H_ 1 

#include <map>
#include <set>
#include <vector>
#include <string>

struct coff_export_symbol
{
	std::string symbol;
};

class image_section_helper
{
	IMAGE_SECTION_HEADER * mh;
	byte * mbase;
	int mid;
	unsigned long start, end;
public:
	image_section_helper() {};
	image_section_helper(IMAGE_SECTION_HEADER * h, int id, byte * base)
		: mh(h), mbase(base + h->PointerToRawData), mid(id),
		  start(h->VirtualAddress), end(h->VirtualAddress + h->SizeOfRawData - 1)
	{}

	byte * get_rva(unsigned long addr);
	int section_id();
	unsigned long dumpsize();
	unsigned long summarysize(unsigned long alignment);
	std::string name() const;
	unsigned long characteristics() const;
private:
	bool uses_virtual_size();
};

typedef enum identify_ {
	IDENTIFY_UNKNOWN,
	IDENTIFY_OBJ,
	IDENTIFY_ANON_OBJ,
	IDENTIFY_LIB,
	IDENTIFY_PE,
	IDENTIFY_OLDDOS,
};

struct cofpcdat
{
	int ident_type;
	struct rvarangebound
	{
		bool start;
		int  sectionid;
		IMAGE_SECTION_HEADER * sectionheader;
	};
	typedef std::map<unsigned long, rvarangebound> rvaranges; 
	struct datadirectory
	{
		int ddid;
		IMAGE_DATA_DIRECTORY * pdd;
	};

	std::map<unsigned long, std::string> maliases;

	/*
	 * Preprocess the symbol table to find overrides/aliases
	 */
	void preprocess_symtable();
	bool test_symtable_alias(unsigned long idx, std::string & result);
	/*
	 * Dumpbin orders the data directories oddly.  Let's replicate!
	 */
	static std::map<int, int> ddorder();

	struct dd_cmp
	{
		static std::map<int, int> dorder;
		bool operator()(const datadirectory & a, const datadirectory & b) const
		{
			return (dorder[a.ddid] < dorder[b.ddid]);
		}
	};

	typedef std::set<datadirectory, dd_cmp> datadirectoryvec;
	typedef std::map<int, datadirectoryvec> ddowners;
	typedef std::map<std::string, std::vector<image_section_helper> > sectionsnamemap;
	typedef std::map<int, int>		iimap;
	typedef iimap::const_iterator	iimapit;

	byte *			base;
	iimap			comdatrefs;
	rvaranges		m_rvaranges;
	ddowners		m_ddowners;
	sectionsnamemap m_sectbyname;
	IMAGE_SYMBOL *	psym;
	int				nsym;
	byte *			stringtable;
	unsigned long	imagebase;
	unsigned long	alignment;

	cofpcdat() : imagebase(0), alignment(0) {}

	std::string stringtable_string(std::string & ref);
	// add open interval.
	void add_section_information(IMAGE_SECTION_HEADER * im, int sectionid);
	bool test_rva(unsigned long addr);
	int find_rva_section(unsigned long addr);
	image_section_helper get_rva_section(unsigned long addr);
	byte * get_rva(unsigned long addr);
	void add_data_directory(int ddid, IMAGE_DATA_DIRECTORY * pd);
	bool has_data_directories(int sectionid);
	datadirectoryvec & get_data_directories(int sectionid);
	sectionsnamemap & get_sectionsbyname();
};


#endif
