#include "stdafx.h"
#include "hexdump.h"
#include <cctype>
#include <string>
#include <iostream>
#include <sstream>
#include <ctime>
#include "dbghelp.h"
#include <algorithm>


long be2le(long x)
{
	return (x & 0xff)     << 24 |
		   (x & 0xff00)   <<  8 |
		   (x & 0xff0000) >>  8 |
		   x >> 24;
}

unsigned long be2le(unsigned long x)
{
	return (x & 0xff)     << 24 |
		   (x & 0xff00)   <<  8 |
		   (x & 0xff0000) >>  8 |
		   x >> 24;
}


short be2le(short x)
{
	return (x & 0xff << 8) | (x & 0xff00 >> 8);
}

std::string rawbytes(void * start, long bytecount)
{
	switch (bytecount)
	{
	case 2:
		return lpad(hex(*(short*)start), 4, '0');
	case 4:
		return lpad(hex(*(unsigned long*)start), 8, '0');
	default:
		throw std::string("Rawbytes unhandled size");
	}
}

bool goodchar (unsigned char asc)
{
	if (asc >= 0x7f && asc <= 0x9f) return false;
	return (asc >= 32 && asc <= 255);
}

std::string hexdump(void * start, long bytecount, unsigned long base)
{
	byte * data = (byte*)start;
	std::stringstream ss;
	do
	{
		int i;
		byte * data2 = data;
		ss << "  " << lpad(hex(data - start + base), 8, '0') << ": ";
		for (i=0;i<16&&bytecount;++i,--bytecount,++data2)
		{
			if (i==8) ss << "";
			char b[10];
			sprintf(b,"%2.2X ",*data2);
			ss << b;
		}
		if (i<9)  ss << "";
		if (i<16) for (int y=i;y<16;++y) ss << "   "; 
		ss << " ";
		data2 = data;
		for (;i;--i,++data2)
		{
			ss << (goodchar(*data2) ? ((char)*data2) : '.');
		}
		data += 16;
		ss << "\n";
	} while (bytecount);
	return ss.str();
}

std::string hex(unsigned int x)
{
	char foo[1024];
	sprintf (foo, "%X", x);
	return std::string(foo);
}

std::string hex(std::string x)
{
	if (x == "") return "";
	return hex(atoi(x.c_str()));
}

std::string lpad(std::string s, int tot, char pad)
{
	int x = s.size();
	tot -= x;
	std::stringstream ss;
	for (int i=0;i<tot;++i) ss << pad;
	ss << s;
	return ss.str();
}

std::string rpad(std::string s, int tot, char pad)
{
	int x = s.size();
	tot -= x;
	std::stringstream ss;
	ss << s;
	for (int i=0;i<tot;++i) ss << pad;
	return ss.str();
}


std::string str(BYTE * s, int len)
{
	std::string x = std::string((char*)s, len).c_str();
	int pos = x.find_last_not_of(" ");
	if (pos == std::string::npos) return "";
//	return x.substr(0, pos);
	std::string y = x.substr(0, pos+1);
	return y;
}

std::string str(int x)
{
	std::stringstream ss;
	ss << x;
	return ss.str();
}

std::string stamptotime(unsigned int t, bool showzero)
{
	if ((t == 0 && !showzero) || t == 0xffffffff) return "";
	time_t ptime = (time_t)t;
	std::string ret(ctime(&ptime));
	ret.resize(ret.length() - 1);
	return std::string(" ") + ret;
}

std::string majmin(int maj, int mn)
{
	return (str(maj) + ".") + lpad(str(mn),2,'0');
}

std::string hex_range(int start, int size)
{
	if (start == 0) return "";
	std::stringstream ss;
	ss << " (" << lpad(hex(start), 8, '0') << " to " << lpad(hex(start+size-1), 8, '0') << ")";
	return ss.str();
}

std::string tolower(std::string s)
{
	USES_CONVERSION;
	CComBSTR b(s.c_str());
	b.ToLower();
	s = OLE2A(b.m_str);	
//	std::transform(s.begin(), s.end(), s.begin(), tolower);
	return s;
}

void split(std::vector<std::string> & outv, std::string & ins, std::string splitchars)
{
	size_t pos;
	size_t offset = 0;
	while (std::string::npos != (pos = ins.find_first_of(splitchars, offset)))
	{
		outv.push_back(ins.substr(offset, pos - offset));
		offset = pos+1;
	}
	outv.push_back(ins.substr(offset));
}


std::string symbol_name(IMAGE_SYMBOL & s, byte * stringtable)
{
	std::string name;
	if (s.N.Name.Short) 
	{
		name = str(s.N.ShortName, 8);
	}
	else
	{
		name = std::string((char*)stringtable + s.N.Name.Long);
	}
	return name;
}
