#ifndef _PROCESSED_IMPL_H_
#define _PROCESSED_IMPL_H_ 1

#include "processed.h"
#include "cofpcdat.h"
#include <vector>

class dbobj_pe : public dbobj
{
	byte * m_base;
	std::vector<cofpcdat> m_cov;
public:
	dbobj_pe(byte * base) : m_base(base) {}
	std::vector<cofpcdat> & cov() { return m_cov; }
	virtual std::string get_name() const;
	virtual std::string get_display() { return ""; }
};

class dbobj_object : public dbobj
{
	byte * m_base;
	std::vector<coff_export_symbol> m_cev;
	std::vector<cofpcdat> m_cov;
public:
	dbobj_object(byte * base) : m_base(base) {}
	std::vector<cofpcdat> & cov() { return m_cov; }
	std::vector<coff_export_symbol> & cev() { return m_cev; }
	virtual std::string get_name() const;
	virtual std::string get_display() { return ""; }

};

class dbobj_library : public dbobj
{
	byte * m_base;
	std::vector<coff_export_symbol> m_cev;
	std::vector<cofpcdat> m_cov;
public:
	dbobj_library(byte * base) : m_base(base) {}
	std::vector<cofpcdat> & cov() { return m_cov; }
	std::vector<coff_export_symbol> & cev() { return m_cev; }
	virtual std::string get_name() const { return "Library"; }
	virtual std::string get_display() { return ""; }
};


class dbobj_anonymous_object : public dbobj
{
	ANON_OBJECT_HEADER * m_hdr;
public:
	dbobj_anonymous_object(ANON_OBJECT_HEADER * hdr) : m_hdr(hdr) {}
	virtual std::string get_name() const;
	virtual std::string get_display();
};

class dbobj_coff_summary : public dbobj
{
	struct summary_item 
	{
		unsigned long loc;
		std::string name;
	};
	std::vector<summary_item> m_si;
public:
	dbobj_coff_summary() {}
	void add_item(unsigned long loc, const std::string & name)
	{
		summary_item si;
		si.loc = loc;
		si.name = name;
		m_si.push_back(si);
	}
	virtual std::string get_name() const { return "COFF Summary"; }
	virtual std::string get_display();
};


class dbobj_coff_import_obj : public dbobj
{
	byte * m_base;
public:
	dbobj_coff_import_obj(byte * base) : m_base(base) {}
	virtual std::string get_name() const { return "COFF Import Object"; }
	virtual std::string get_display();
};

class dbobj_coff_file : public dbobj
{
	IMAGE_FILE_HEADER * m_hdr;
public:
	dbobj_coff_file(IMAGE_FILE_HEADER * hdr) : m_hdr(hdr) {}
	virtual std::string get_name() const { return "COFF Object"; }
	virtual std::string get_display() { return ""; }
};

class dbobj_coff_header : public dbobj
{
	IMAGE_FILE_HEADER * m_hdr;
public:
	dbobj_coff_header(IMAGE_FILE_HEADER * hdr) : m_hdr(hdr) {}
	virtual std::string get_name() const { return "Header"; }
	virtual std::string get_display();
};

class dbobj_coff_opt_header : public dbobj
{
	IMAGE_OPTIONAL_HEADER32 * m_hdr;
	cofpcdat * m_pcd; // we don't own it!
public:
	dbobj_coff_opt_header(IMAGE_OPTIONAL_HEADER32 * hdr, cofpcdat * pcd) : m_hdr(hdr), m_pcd(pcd) {}
	virtual std::string get_name() const { return "Optional Header"; }
	virtual std::string get_display();
};

class dbobj_container : public dbobj
{
	std::string m_name;
public:
	dbobj_container(std::string name) : m_name(name) {}
	virtual std::string get_name() const { return m_name; }
	virtual std::string get_display() { return ""; }
};

/*
 * This is an ungly way of doing indirections :P
 */
class dbobj_indirection : public dbobj
{
	std::string m_name;
public:
	dbobj_indirection(std::string name) : m_name(name) {}
	virtual std::string get_name() const { return m_name; }
	virtual bool is_indirection(std::string & key) { key = m_name; return true; }
	virtual std::string get_display() { return ""; }
};


class dbobj_coff_section : public dbobj
{
	IMAGE_SECTION_HEADER *m_ph;
	int m_id;
	cofpcdat * m_pcd; 
	image_section_helper m_sh;
	
	std::string warning;
public:
	dbobj_coff_section(IMAGE_SECTION_HEADER * ph, int id, cofpcdat * pcd, image_section_helper & sh) :
	  m_ph(ph), m_id(id), m_pcd(pcd), m_sh(sh) {}

	void add_warning(std::string & s) { warning = s; }

	int get_id() { return m_id; }
	IMAGE_SECTION_HEADER * get_section_header() { return m_ph; }
	cofpcdat & get_cd() { return *m_pcd; }
	image_section_helper & get_sh() { return m_sh; }
	  
	virtual std::string get_name() const;
	virtual std::string get_display();
};

class dbobj_section_header : public dbobj
{
	dbobj_coff_section * m_s;
public:
	dbobj_section_header(dbobj_coff_section * s) :
	  m_s(s) {}
	virtual std::string get_name() const { return "Header"; }
	virtual std::string get_display();
};

class dbobj_section_data : public dbobj
{
	dbobj_coff_section * m_s;
public:
	dbobj_section_data(dbobj_coff_section * s) :
	  m_s(s) {}
	virtual std::string get_name() const { return "Data"; }
	virtual std::string get_display();
};

class dbobj_section_linker_directives : public dbobj
{
	dbobj_coff_section * m_s;
public:
	dbobj_section_linker_directives(dbobj_coff_section * s) :
	  m_s(s) {}
	virtual std::string get_name() const { return "Linker Directives"; }
	virtual std::string get_display();
};

class dbobj_data_directory : public dbobj
{
	byte * m_start;
	int m_size;
	cofpcdat * m_pcd;
public:
	byte * get_start() { return m_start; }
	cofpcdat & get_cd() { return *m_pcd; }
	int get_size() { return m_size; }
	dbobj_data_directory(byte * start, int size, cofpcdat * pcd) : m_start(start), m_size(size), m_pcd(pcd) {}
	dbobj_data_directory(byte * start, int size) : m_start(start), m_size(size), m_pcd(NULL) {}
};

class dbobj_bound_import_directory : public dbobj_data_directory
{
public:
	dbobj_bound_import_directory(byte * start, int size) : dbobj_data_directory(start, size) {}
	virtual std::string get_name() const { return "Bound Imports"; }
	virtual std::string get_display();
};

class dbobj_export_directory : public dbobj_data_directory
{
public:
	dbobj_export_directory(byte * start, int size, cofpcdat * pcd) : dbobj_data_directory(start, size, pcd) {}
	virtual std::string get_name() const { return "Exports"; }
	virtual std::string get_display();
};

class dbobj_import_directory : public dbobj_data_directory
{
public:
	dbobj_import_directory(byte * start, int size, cofpcdat * pcd) : dbobj_data_directory(start, size, pcd) {}
	virtual std::string get_name() const { return "Imports"; }
	virtual std::string get_display();
};

class dbobj_exception_directory : public dbobj_data_directory
{
public:
	dbobj_exception_directory(byte * start, int size, cofpcdat * pcd) : dbobj_data_directory(start, size, pcd) {}
	virtual std::string get_name() const { return "Exceptions"; }
	virtual std::string get_display();
};

class dbobj_reloc_directory : public dbobj_data_directory
{
	image_section_helper m_sh;
public:
	dbobj_reloc_directory(byte * start, int size, cofpcdat * pcd, image_section_helper & sh) : dbobj_data_directory(start, size, pcd), m_sh(sh) {}
	virtual std::string get_name() const { return "Relocations"; }
	virtual std::string get_display();
};

class dbobj_debug_directory : public dbobj_data_directory
{
public:
	dbobj_debug_directory(byte * start, int size, cofpcdat * pcd) : dbobj_data_directory(start, size, pcd) {}
	virtual std::string get_name() const { return "Debug Data"; }
	virtual std::string get_display();
};

class dbobj_tls_directory : public dbobj_data_directory
{
	image_section_helper m_sh;
public:
	dbobj_tls_directory(byte * start, int size, cofpcdat * pcd, image_section_helper & sh) : dbobj_data_directory(start, size, pcd), m_sh(sh) {}
	virtual std::string get_name() const { return "TLS"; }
	virtual std::string get_display();
};

class dbobj_load_config_directory : public dbobj_data_directory
{
public:
	dbobj_load_config_directory(byte * start, int size, cofpcdat * pcd) : dbobj_data_directory(start, size, pcd) {}
	virtual std::string get_name() const { return "Load Config"; }
	virtual std::string get_display();
};

class dbobj_delay_import_directory : public dbobj_data_directory
{
	image_section_helper m_sh;
public:
	dbobj_delay_import_directory(byte * start, int size, cofpcdat * pcd, image_section_helper & sh) : dbobj_data_directory(start, size, pcd), m_sh(sh) {}
	virtual std::string get_name() const { return "Delayed Imports"; }
	virtual std::string get_display();
};

class dbobj_com_descriptor_directory : public dbobj_data_directory
{
public:
	dbobj_com_descriptor_directory(byte * start, int size, cofpcdat * pcd) : dbobj_data_directory(start, size, pcd) {}
	virtual std::string get_name() const { return "COM Descriptor"; }
	virtual std::string get_display();
};

class dbobj_section_relocations : public dbobj
{
	dbobj_coff_section * m_s;
public:
	dbobj_section_relocations(dbobj_coff_section * s) :
	  m_s(s) {}
	virtual std::string get_name() const { return "Relocations"; }
	virtual std::string get_display();
};

class dbobj_section_linenumbers : public dbobj
{
	dbobj_coff_section * m_s;
public:
	dbobj_section_linenumbers(dbobj_coff_section * s) :
	  m_s(s) {}
	virtual std::string get_name() const { return "Line Numbers"; }
	virtual std::string get_display();
};


class dbobj_image_archive_member_header : public dbobj
{
	IMAGE_ARCHIVE_MEMBER_HEADER * m_ph;
	std::string m_longname;
	unsigned int m_offset;
	int m_idx;
	enum 
	{
		dbo_iamh_linker_member,
		dbo_iamh_longnames_member,
		dbo_iamh_other_member
	} mem_type;
public:
	dbobj_image_archive_member_header(IMAGE_ARCHIVE_MEMBER_HEADER * ph, std::string slongname, unsigned int offset, int idx) :
	  m_ph(ph), m_longname(slongname), m_offset(offset), m_idx(idx) 
	{
		if (!memcmp(ph->Name, IMAGE_ARCHIVE_LINKER_MEMBER, 16))
		{
			mem_type = dbo_iamh_linker_member;
		}
		else if (!memcmp(ph->Name, IMAGE_ARCHIVE_LONGNAMES_MEMBER, 16))
		{
			mem_type = dbo_iamh_longnames_member;
		}
		else
		{
			mem_type = dbo_iamh_other_member;
		}
	};
	virtual std::string get_name() const;
	virtual std::string get_display();
};


class dbobj_iamh_symbols : public dbobj
{
	byte * m_data;
public:
	dbobj_iamh_symbols(byte * data) :
	  m_data(data) {}

	virtual std::string get_name() const { return "Symbols"; }
	virtual std::string get_display();
};


class dbobj_iamh_offsets : public dbobj
{
	byte * m_data;
public:
	dbobj_iamh_offsets(byte * data) : m_data(data) {}

	virtual std::string get_name() const { return "Offsets"; }
	virtual std::string get_display();
};


class dbobj_coff_exports : public dbobj
{
	std::vector<coff_export_symbol> & m_cev;
public:
	dbobj_coff_exports(std::vector<coff_export_symbol> & cev) : m_cev(cev) {}

	virtual std::string get_name() const { return "COFF exports"; }
	virtual std::string get_display();
};

class dbobj_coff_symbol_table : public dbobj
{
	cofpcdat & m_cd;
public:
	dbobj_coff_symbol_table(cofpcdat & cd) : m_cd(cd) {}

	virtual std::string get_name() const { return "COFF Symbol table"; }
	virtual std::string get_display();
};


#endif
